//*****************************************************************************
// File Name	: 'lcd_lib_STC.c'
// Title		: Library C File for ST7036 COntroller
// Author		: Sahil Potnis, Cornell University
// Created		: 11-14-14
// Version		: 1.0
// Target MCU	: Atmel AVR series
// Operational Mode : 5V 4 Bit interface for AVR-GCC and EZ DOGM163W-A LCD
//*****************************************************************************

#include "lcd_lib_STC.h"
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

//******************************************************************************
//writes 8 bit to the ST7036 in 2 nibbles (2 x 4bit)
void write(char data)
{
	char port_buffer = LCD_PORT;
	
	//write the high nibble first...
	char nibble = (data & 0xF0) >> 4;
	LCD_PORT |= nibble;
	LCD_PORT |= (1 << PIN_ENABLE);
	_delay_ms(1);
	
	LCD_PORT = port_buffer;
	_delay_ms(1);
	
	//...then the low nibble
	nibble = data & 0x0F;
	LCD_PORT |= nibble;
	LCD_PORT |= (1 << PIN_ENABLE);
	_delay_ms(1);
	
	LCD_PORT = port_buffer;
}
//******************************************************************************
//******************************************************************************
void write_instruction(char instruction)
{
	//RS low = instruction
	LCD_PORT &= ~(1 << PIN_RS);
	write(instruction);
}
//******************************************************************************
void write_data(char data)
{
	//RS high = data
	LCD_PORT |= (1 << PIN_RS);
	write(data);
}
//******************************************************************************
void set_cursor(char row, char column)
{
	write_instruction(CHARACTER_BUFFER_BASE_ADDRESS + row * CHARACTERS_PER_ROW + column);
}
//******************************************************************************
void write_string(char* string)
{
	while (*string)
	{
		write_data(*string);
		*string++;
    }
}
//******************************************************************************
void initialize_display()
{
	//wait for a short period, maybe the voltage needs to stabilize first
	_delay_ms(50);
	
	//initialize 4 bit mode
	write_instruction(INSTRUCTION_FUNCTION_SET_INIT_0);
	write_instruction(INSTRUCTION_FUNCTION_SET_INIT_1);
	write_instruction(INSTRUCTION_FUNCTION_SET_INIT_2);
	
	//initialize everything else
	write_instruction(INSTRUCTION_BIAS_SET);
	write_instruction(INSTRUCTION_POWER_CONTROL);
	write_instruction(INSTRUCTION_FOLLOWER_CONTROL);
	write_instruction(INSTRUCTION_CONTRAST_SET);
	write_instruction(INSTRUCTION_INSTRUCTION_SET_0);
	write_instruction(INSTRUCTION_DISPLAY_ON);
	write_instruction(INSTRUCTION_CLEAR_DISPLAY);
	write_instruction(INSTRUCTION_ENTRY_MODE);
}
//******************************************************************************
void LCD_clr(void)
{

write_instruction(INSTRUCTION_CLEAR_DISPLAY);


}
//******************************************************************************
void LCDcursor_ON(void)
{
write_instruction(INSTRUCTION_CURSOR_ON);   

}
//******************************************************************************
void LCDcursor_OFF(void)	//turns OFF cursor
{
	write_instruction(INSTRUCTION_CURSOR_OFF);
}
//******************************************************************************
void LCDcursor_Left(uint8_t n)	//Moves cursor by n poisitions left
{
	for (uint8_t i=0;i<n;i++)
	{
		write_instruction(INSTRUCTION_CURSOR_RIGHT);
	}
}
//******************************************************************************
void LCDcursor_Right(uint8_t n)	//Moves cursor by n poisitions left
{
	for (uint8_t i=0;i<n;i++)
	{
		write_instruction(INSTRUCTION_CURSOR_LEFT);
	}
}
//******************************************************************************
void LCDcursor_blink(void)
{

write_instruction(INSTRUCTION_CURSOR_BLINK);

}
//******************************************************************************

void LCD_string(uint8_t* data, uint8_t nBytes)	//Outputs string to the LCD
{
register uint8_t i;

	// check to make sure we have a good pointer
	if (!data) return;

	// print data
	for(i=0; i<nBytes; i++)
	{
		write_data(data[i]);
	}
}
//******************************************************************************

void CopyStringto_LCD(const uint8_t *FlashLoc, uint8_t x, uint8_t y)
{
	uint8_t i;
	set_cursor(x,y);
	for(i=0;(uint8_t)pgm_read_byte(&FlashLoc[i]);i++)
	{
		write_data((uint8_t)pgm_read_byte(&FlashLoc[i]));
	}
}
//******************************************************************************
